Imports System.Xml.Serialization

<XmlRoot("configurazione")> _
Public Class Configuration

#Region " Membri privati "

    Private strHost As String
    Private intPort As Integer
    Private strStartPath As String
    Private uspUsers As New List(Of UserPass)
    Private intTimesUsed As Integer

#End Region

    'Le propriet pubbliche sono l'oggetto della serializzazione
#Region " Propriet pubbliche "

    'Per questa propriet abbiamo impostato che deve essere
    'serializzato come un elemento chiamato Server
    <XmlElement("Server")> _
    Public Property Host() As String
        Get
            Return strHost
        End Get
        Set(ByVal value As String)
            strHost = value
        End Set
    End Property

    'Nessuna indicazione sulla serializzazione, verr trattato
    'come un elemento chiamato Port
    Public Property Port() As Integer
        Get
            Return intPort
        End Get
        Set(ByVal value As Integer)
            intPort = value
        End Set
    End Property

    'Nessuna indicazione sulla serializzazione, verr trattato
    'come un elemento chiamato TimesUsed
    Public Property TimesUsed() As Integer
        Get
            Return intTimesUsed
        End Get
        Set(ByVal value As Integer)
            intTimesUsed = value
        End Set
    End Property

    'Gli oggetti di tipo System.Uri non possono essere serializzati
    'perch non dispongono di un costruttore senza parametri, per
    'questo bisogna indicare che non venga serializzato tramite
    'l'attributo XmlIgnore
    <XmlIgnore()> _
    Public ReadOnly Property Uri() As Uri
        Get
            Return New Uri(String.Format("ftp://{0}:{1}/", Me.Host, Me.Port))
        End Get
    End Property

    'Lista: verr serializzata come un elemento Users
    'dotato di una seria di sotto-elementi UserPass
    Public ReadOnly Property Users() As List(Of UserPass)
        Get
            Return uspUsers
        End Get
    End Property

#End Region

    'Semplice classe che raggruppa nome utente e password
    'per mostrare come vengono serializzate membri complessi
    '(ovvero non semplici Integer, String o altro), in questo
    'caso tramite la propriet Users che restituisce una lista
    'di istanza della classe che segue.
    Public Class UserPass
        Private strUser As String
        Private strPass As String

        Public Sub New(ByVal strUser As String, ByVal strPass As String)
            Me.strUser = strUser
            Me.strPass = strPass
        End Sub

        'Per serializzare un oggetto  sempre necessario che
        'la classe disponga di un costruttore senza parametri
        Public Sub New()
        End Sub

        'Indica che questa propriet deve essere serializzata
        'come un attributo dell'elemento che lo contiene (in 
        'questo caso UserPass). L'attributo si chiamer username
        <XmlAttribute("username")> _
        Public Property User() As String
            Get
                Return strUser
            End Get
            Set(ByVal value As String)
                strUser = value
            End Set
        End Property

        'Indica che questa propriet deve essere serializzata
        'come un attributo dell'elemento che lo contiene (in 
        'questo caso UserPass). L'attributo si chiamer password
        <XmlAttribute("password")> _
        Public Property Pass() As String
            Get
                Return strPass
            End Get
            Set(ByVal value As String)
                strPass = value
            End Set
        End Property

    End Class

    'Semplice scorciatoia per il metodo Save statico (shared) per 
    'serializzare su disco l'oggetto corrente
    Public Sub Save(ByVal strFile As String)
        Save(strFile, Me)
    End Sub

#Region " Funzioni statiche (Shared) "

    'Serializza sul file specificato l'oggetto Configuration
    'passato come argomento
    Public Shared Sub Save(ByVal strFile As String, ByVal cfg As Configuration)
        Dim wrtFile As New IO.FileStream(strFile, IO.FileMode.Create)
        Dim xsrSerializer As New XmlSerializer(GetType(Configuration))
        xsrSerializer.Serialize(wrtFile, cfg)
        wrtFile.Close()
    End Sub

    'Crea un oggetto Configuration con dei valori di default
    'e lo serializza su file
    Public Shared Sub CreateDefaultConfig(ByVal strFile As String)
        Dim cfgNew As New Configuration
        cfgNew.Host = "ftp.test.com"
        cfgNew.Port = 21
        cfgNew.Users.Add(New UserPass("anonymous", "email@address.com"))
        cfgNew.Users.Add(New UserPass("Carl", "carlitos"))
        cfgNew.strStartPath = "/cartella/"
        cfgNew.TimesUsed = 0
        Save(strFile, cfgNew)
    End Sub

    'Carica in memoria un oggetto Configuration precedentemente 
    'serializzato su file
    Public Shared Function LoadFromFile(ByVal strFile As String) As Configuration
        Dim rdr As New IO.FileStream(strFile, IO.FileMode.Open)
        Dim xsrDeserializer As New XmlSerializer(GetType(Configuration))
        Dim cfgResult As Configuration = _
            CType(xsrDeserializer.Deserialize(rdr), Configuration)
        rdr.Close()
        Return cfgResult
    End Function

#End Region

End Class
